#!/usr/bin/python
# -*- coding: UTF-8 -*-

import os
import re
import sys
from iniconfigloader import IniConfigLoader
from xmlconfigwriter import XmlConfigWriter

print "Старт миграции настроек оборудования 4.5.7 -> 4.6"
ini = IniConfigLoader()
xml = XmlConfigWriter()
frs = {} # добавленные фр-ы

def addFr(section):
    """
    Добавляет ФР из секции section
    """

    if section in frs:
        return

    options = {}
    libName = None
    driver = ini.getString(section, "driver")
    if driver == "dummy":
        libName = 'frdummy'
    elif driver == "fileprint":
        options = {
                    "fileName": ini.getString(section, "fileName", '/dev/null'),
                    "baudWidth": ini.getString(section, "width", '36')
                    }
        libName = 'frfileprint'
    elif driver == "shtrihm":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")},
                    "baudrate": ini.getString(section, "baudrate", "115200"),
                    "accessCode": ini.getString(section, "accesscode"),
                    "serviceCode": ini.getString(section, "servicecode"),
                    "denyBeeps":  {
                        "type": "bool",
                        "value": ini.getString(section, "denyBeeps")},
                    "denyCountersCompare": {
                        "type": "bool",
                        "value": ini.getString(section, "denyCountersCompare")},
                    "useCutter": {
                        "type": "bool",
                        "value": ini.getString(section, "useCutter")},
                    "cutAfterChecksClosing": {
                        "type": "bool",
                        "value": ini.getString(section, "cutAfterChecksClosing")},
                    "useBandWidthFromFont": {
                        "type": "bool",
                        "value": ini.getString(section, "useBandWidthFromFont")}
                    }
        libName = 'frshtrihm'
    elif driver == "sp101":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'frsp101'
    elif driver == "fp410":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")},
                    "accessCode": ini.getString(section, "accesscode"),
                    "allowPrintFiscalInfo": {
                        "type": "bool",
                        "value": ini.getString(section, "allowPrintFiscalInfo")},
                    "allowChangeLineSpacing": {
                        "type": "bool",
                        "value": ini.getString(section, "allowChangeLineSpacing")}
                    }
        libName = 'frfp410'
    elif driver == "atol":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")},
                    "baudrate": ini.getString(section, "baudrate", "115200"),
                    "accessCode": ini.getString(section, "accesscode"),
                    "userCode": ini.getString(section, "usercode"),
                    "denyBeeps": {
                        "type": "bool",
                        "value": ini.getString(section, "denyBeeps")}
                    }
        libName = 'fratol'
    elif driver == "epson":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")},
                    "baudrate": ini.getString(section, "baudrate", "115200"),
                    "timeout": ini.getString(section, "timeout"),
                    "width": ini.getString(section, "width"),
                    "codePage": ini.getString(section, "codepage")
                    }
        libName = 'frepson'
    elif driver == "pirit":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'frpirit'
    elif driver == "envd":
        dev = ini.getString(section, "device")
        if dev in ini.sections():
            if dev not in frs:
                addFr(dev)
            options = {
                        "serialObj": {
                            "type": "deviceRef",
                            "value": frs[dev]},
                        "printerSettings": {
                            "type": "bool",
                            "value": ini.getString(section, "printsettings")},
                        "registerMoneyIn": {
                            "type": "bool",
                            "value": ini.getString(section, "registerMoneyIn")}
                        }
            libName = 'frenvd'

    if libName:
        print "Добавление ФР '%s'" % driver
        m = re.search('^HW.FR(.([0-9]+))?$', section, re.I)
        deviceId = '0' if m.group(2) == None else m.group(2)
        options['deviceId'] = deviceId
        frs[section] = xml.addDevice(libName, options)

def addDrawer(section):
    """
    Добавляет денежный ящик из секции section
    """
    print "Начинаем добавление ДЯ"
    mode = ini.getString(section, "mode")
    libName = None
    options = {}
    if mode == 'fr':
        # Для всех фр-ов
        print "Добавление ДЯ. Режим ФР"
        frsSorted = frs.keys()
        frsSorted.sort()
        if frsSorted:
            fr = frs[frsSorted[0]]
            print "Добавление ДЯ для ФР '%s'" % fr
            options = {
                    "fiscalRegisterObj": {
                        "type": "deviceRef",
                        "value": fr}
                }
            libName = 'cashdrawerfr'
    elif mode == 'ibm':
        libName = 'ibmcashdrawer'
    elif mode == 'fujitsu':
        print "Денежный ящик 'fujitsu' не поддерживается в 4.6"

    if libName:
        print "Добавление денежного ящика '%s'" % mode
        xml.addDevice(libName, options)
    
    print "Закончили добавление ДЯ"

def addCustomerDisplay(section):
    """
    Добавляет дисплей покупателя из секции section
    """
    protocol = ini.getString(section, "protocol")
    device = ini.getString(section, "device")
    libName = None
    options = {}
    if protocol == "ibmusb":
        print "Дисплей покупателя 'ibmusb' не поддерживается в 4.6"
    elif protocol == "epson" and device == "posiflexusb":
        libName = 'cdposiflexusb'
    elif protocol == "direct" or protocol == "epson":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")},
                    "encoding": ini.getString(section, "encoding")
                    }
        if protocol == "epson":
            options.update( {"switchCodec": "true"} )
        libName = 'cdepson'
    elif protocol == "fujitsu":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'cdfujitsu'
    elif protocol == "firich":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'cdfirich'
    elif protocol == "wincor":
        print "Дисплей покупателя 'wincor' не поддерживается в 4.6"
    elif protocol == "posiflexusb":
        print "Дисплей покупателя 'posiflexusb' не поддерживается в 4.6"
    elif protocol == "datecs":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'cddatex'
    elif protocol == "lightpos":
        print "Дисплей покупателя 'lightpos' не поддерживается в 4.6"
    elif protocol == "ibm":
        libName = 'cdibm'
    elif protocol == "posiflexpd320":
        options = {
                    "serialObj": {
                        "type": "serialRef",
                        "value": ini.getString(section, "device")}
                    }
        libName = 'cdposiflex'

    if libName:
        print "Добавление дисплея покупателя '%s'" % protocol
        xml.addDevice(libName, options)

# поиск интересующих нас секций с оборудованием
for section in ini.sections():
    if re.match('^HW.FR(.[0-9]+)?$', section, re.I):
        addFr(section)
    if re.match('^HW.CustomerDisplay', section, re.I):
        addCustomerDisplay(section)

# Обязательно после добавления всех ФР
print "Поиск настроенных ДЯ"
for section in ini.sections():    
    if re.match('^HW.Drawer', section, re.I):
        print "Найден ДЯ"
        addDrawer(section)

print "Миграция настроек успешно завершена"
